use std::fmt::{Display, Formatter};
use ash::vk::QueueFlags;
use winit::application::ApplicationHandler;
use winit::event::WindowEvent;
use winit::event_loop::ActiveEventLoop;
use winit::window::{WindowId};
use crate::display::window::Window;
use crate::vulkan::{VkInstance, VkPhysicalDevice};

pub struct App {
    window: Window,
    instance: Option<VkInstance>
}

impl App {
    pub fn new(window: Window) -> Self {
        Self {
            window,
            instance: None
        }
    }

    fn init_vulkan(&mut self) {
        let required_extensions = self.window
            .required_extensions()
            .expect("Unable to get required required extensions");

        log::info!("Initializing Vulkan instance");
        let instance = VkInstance::new(&required_extensions);
        log::info!("Vulkan instance created");
        log::info!("\t{}", instance);

        let surface = instance.create_surface(&self.window)
            .expect("Unable to create surface");

        let mut physical_devices = instance.get_physical_devices();
        physical_devices.sort_by(|a, b| b.priority().cmp(&a.priority()));

        let (physical_device, queue_family_index) = physical_devices
            .iter()
            .find_map(|physical_device| {
                physical_device.queue_family_properties
                    .iter()
                    .enumerate()
                    .find_map(|(index, queue_family_property)| {
                        if surface.physical_device_queue_supported(physical_device, index as u32).unwrap_or(false) {
                            Some((physical_device, index as u32))
                        } else {
                            None
                        }
                    })
            })
            .expect("Unable to find suitable device");


        self.instance = Some(instance);

    }
}

impl ApplicationHandler for App {
    fn resumed(&mut self, event_loop: &ActiveEventLoop) {
        self.window.create_window(event_loop)
            .map_err(|err| format!("Failed to create window: {}", err))
            .unwrap();

        self.init_vulkan();
    }

    fn window_event(&mut self, event_loop: &ActiveEventLoop, id: WindowId, event: WindowEvent) {
        match event {
            WindowEvent::CloseRequested => {
                log::info!("The close button was pressed; stopping");
                event_loop.exit();
            }
            _ => self.window.window_event(event_loop, id, event),
        }
    }
}
